Package: gcTensor
Type: Package
Title: Generalized Coupled Tensor Factorization
Version: 1.0.0
Authors@R: c(person("Koki", "Tsuyuzaki", role = c("aut", "cre"),
                      email = "k.t.the-answer@hotmail.co.jp"))
Suggests: testthat
Depends: R (>= 4.1.0)
Imports: rTensor, einsum
Description: Multiple matrices/tensors can be specified and decomposed simultaneously by Probabilistic Latent Tensor Factorisation (PLTF). See the reference section of GitHub README.md <https://github.com/rikenbit/gcTensor>, for details of the method.
License: MIT + file LICENSE
URL: https://github.com/rikenbit/gcTensor
Author: Koki Tsuyuzaki [aut, cre]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Built: R 4.4.0; ; 2024-05-06 03:29:59 UTC; unix
